#include <iostream>
#include <string>
#include <valarray>
#include <vector>
#include <fstream>
#include <list>
#include <cmath>
#include <sstream>
#include <cstdio>

using namespace std;
typedef double REAL;
typedef valarray <REAL> COORDINATE;
// ------------------------ global variables -------------------------------
static char const * help =
"interpolate_cube 1.0 (c) 2001 Felix Koehler\n"
"Usage:  AICD-criticalpoints <...>\n"
"Description: Reads the numerical function for the anisotropy of the induced\n"
"current density (AICD) in form of a cube file and evaluates the critical\n"
"points\n"
"Options:\n"
"   -i<string>: name of input cube file\n"
"   -h         display this help\n"
;

vector < vector < vector <REAL> > > AICD; // original values: 3D Array
COORDINATE llb (0.0, 3); // coordinates of lower left bottom of cube
COORDINATE urt (1.0, 3); // coordinates of upper right top of cube
COORDINATE grid (0.1, 3); // grid spacing 
valarray <int > ppa (0, 3); // number of points per axis
int nAtoms; // number of atoms in cube file
string input_filename; // name of input cube file
string output_filename; // name of output cube file


// ------------------------ prototypes ---------------------------------------
void parse (int argc, char* argv[]);
void read_cube_file ();
REAL interpolate_AICD (COORDINATE);
void test_interpolate ();


// ------------------------ code ---------------------------------------
int main(int argc, char* argv[])
{
	try
	{
		parse (argc, argv);				// read the files
		read_cube_file();
		test_interpolate ();
	}
	catch (const char* errstr)
	{
		cerr << errstr << endl;
	}
	catch (exception& e)
	{
		cerr << "Standardbibliotheksexception:" << e.what() << endl;
	}
	catch (...)
	{
		cerr << "Error in interpolate_cube" << endl;
	}
	return 0;
}

void parse (int argc, char* argv[])
{
	while (argc>1 && argv[1][0]=='-')
	{
		switch (argv[1][1])
		{
		case '?': case 'h': cout << help; exit(1); break;
		case 'i': input_filename=string(argv[1]+2); break;
		case 'o': output_filename=string(argv[1]+2); break;
		default:
			throw "Unknown option "+string(argv[1]);
			break;
    }
		argv++;
		argc--;
	}
	if (input_filename=="") throw "Error: Filenames required\n";
	if (output_filename=="") {
		output_filename = input_filename.substr(0, input_filename.rfind (".cub")) + ".cub";
	}
}

void read_cube_file ()
// --------------------- function read_cube_file () ------------------
// global Variables used for read:
//
//   input_filename : Filename of the cube to be read. Cube must contain density data,
//                    no orbital data allowed
//
// global Variables changed:
//
//   AICD[[[]]] : the scalar AICD vector field is allocated and initialized
//   ppa []     : points per axis
//   grid []    : gridsize of one volume element
//   llb []     : lower left bottom of cube
//   urt []     : upper right top of cube
{
	REAL real_dummy;
	int i;
	string line;

	ifstream in (input_filename.c_str());
	getline (in, line);

	in >> nAtoms >> llb[0] >> llb[1] >> llb[2];
	in >> ppa[0] >> grid[0] >> real_dummy >> real_dummy;
	in >> ppa[1] >> real_dummy >> grid[1]>> real_dummy;
	in >> ppa[2] >> real_dummy >> real_dummy >> grid[2];
	urt = (ppa - 1) * grid + llb;
	
	// read the atomic coordinate entries
	for (i=0; i<abs(nAtoms); ++ i) {
		getline (in, line);
	}
	getline (in, line);

	vector <REAL> Z (ppa[0], 0.0);
	vector <vector <REAL> > Y (ppa[1], Z);
	AICD.resize (ppa[2], Y);

	// cubedata (z is the innermost loop)
	int x,y,z;
	for (x = 0; x<ppa[0]; ++x)
		for (y = 0; y<ppa[1]; ++y)
			for (z = 0; z<ppa[2]; ++z) {
				if (!in || in.eof()) throw "Error reading original cube values.";
				in >> AICD[x][y][z];
			}
	in.close();
}

// The following isn't nice, but there seems to be no standard round
template <class R>
inline int koehli_round (R r) { return int(r+0.5); }

REAL interpolate_AICD (COORDINATE p)
// --------------------- function read_cube_file () ------------------
// global Variables used for read:
//
//   AICD[[[]]] : the scalar AICD vector field is allocated and initialized
//   ppa []     : points per axis
//   grid []    : gridsize of one volume element
//   llb []     : lower left bottom of cube
//   urt []     : upper right top of cube
//
// Algorithm:
//
// 1. find interpolation points: Use nearest volumeelement.
// If point p is within the cube this means there will be a volumeelement or a plane
// or a line or a single point that contains p.
// 

{
	COORDINATE g = ( p - llb ) / grid; // Grid Coordinates in the grid space of AICD
	const REAL SMALLFLOAT=11.11E-11;
	int i;
	for (i=0; i<=2; ++i) {
		
	
	list <int> xgrid, ygrid, zgrid;
	if (fabs (koehli_round(g[0])-g[0]) <= SMALLFLOAT) xgrid.push_back(koehli_round(ox));
		else {
			xgrid.push_back((int)ceil(g[0]));
			xgrid.push_back((int)floor(g[0]));
	} ;
	if (fabs (koehli_round(g[1])-g[1]) <= SMALLFLOAT) ygrid.push_back(koehli_round(oy));
		else {
			ygrid.push_back((int)ceil(g[1]));
			ygrid.push_back((int)floor(g[1]));
	};
	if (fabs (koehli_round(g[2])-g[2]) <= SMALLFLOAT) zgrid.push_back(koehli_round(oz));
		else { 
			zgrid.push_back((int)ceil(g[2]));
			zgrid.push_back((int)floor(g[2]));
	};

		// Generate list of interpolationpoints
		list <s_point> iplist; // a list of interpolation points
		s_point p; // a point object to be added to the list of interpolation points
		list<int>::iterator xit,yit,zit;
		for (xit=xgrid.begin(); xit!=xgrid.end(); ++xit) {
			for (yit=ygrid.begin(); yit!=ygrid.end(); ++yit) {
				for (zit=zgrid.begin(); zit!=zgrid.end(); ++zit) {
					p.x = *xit;
					p.y = *yit;
					p.z = *zit;
					iplist.push_back (p);
				}
			}
		}


		// Finally : do interpolation with list of gridpoints
		REAL w; // weight for each interpolationpoint
		REAL sum_of_w = 0;
		REAL sum_of_w_times_value = 0;
		list <s_point>::iterator iter;
		int x,y,z;
		for (iter=iplist.begin(); iter != iplist.end(); ++iter) {
			x=(*iter).x;
			y=(*iter).y;
			z=(*iter).z;
			w =  (1.0 - fabs (x - ox)) * (1.0 - fabs (y - oy)) * (1.0 - fabs (z - oz));
			sum_of_w += w;

/*	// index calculation for x being the inner loop
			int index = z*Nx*Ny + y*Nx + x;
*/

	// index calculation for z being the inner loop
			int index = x*Ny*Nz + y*Nz + z;

			if (index >= AICD.size())
				throw "index out of bounds for nv";
			sum_of_w_times_value += (w * AICD[index]);
		}
		nv[i] = sum_of_w_times_value / sum_of_w;
	}
}

void test_interpolate ()
{
}
